# Driplet Module

A Drupal module that provides pushing messages to the frontend using [https://github.com/make0x20/driplet](Driplet) (using WebSockets). It enables real-time notifications and messaging to specific users or roles. The module depends on [https://github.com/make0x20/driplet-php](Driplet PHP library).

## Use Cases

- Sending real-time notifications to users
- Live update on upvotes, likes, etc.
- Notifying users of new content or updates in real-time
- Alerting users of system events or changes
- Real-time chat applications between users
- Any other use case where we need to push messages to the frontend in real-time

## Installation

1. Install the module as you would any other Drupal module
2. Configure the module at `/admin/config/services/driplet`
3. Set up your endpoint configuration with:
   - API Secret (for message signing)
   - JWT Secret (for client authentication)

## Integration

The module works in conjunction with Driplet - a Go-based microservice. Ensure Driplet is properly configured and running before using the module.


## Configuration

Example configuration in your `settings.php` or `settings.local.php`:

```php
$config['driplet.settings']['driplet_endpoint'] = 'https://your-driplet-server:4719/api/default/message';
$config['driplet.settings']['driplet_api_secret'] = 'your-api-secret';
$config['driplet.settings']['driplet_jwt_secret'] = 'your-jwt-secret';
```

## How it works

A backend service (e.g. Drupal) can push a message to Driplet microservice, which will then push the message to the frontend clients. The message can be targeted to specific users, roles, or topics. The frontend clients can subscribe to topics and receive messages in real-time.

## In more detail and usage

Driplet can authenticate users based on their JWT tokens which Driplet module generates on (/api/driplet/jwt) endpoint. The JWT includes user ID and user roles which can be used as targeting criteria for messages. For instance, it is possible to send a message only to specific roles, uids, or a combination of both. It's also possible to invert the logic and specify exclusion criteria.

There is also a concept of topics which can be used to group messages. For instance, on a specific page, a frontend user can subscribe only to a topic relevant to that page. This way, the user will receive only messages relevant to that topic. It's possible to subscribe to multiple topics at once, and handle each topic separately.

### Example message structure

Message will be received by all `administrator` users that are subscribed to `some-topic` topic, except the ones with uid 1, 2, or 3.

```json
{
  "message": {
    "some": "some",
    "custom": "custom",
    "keys": "data"
  },
  "target": {
    "include": {
      "roles": ["administrator"]
    },
    "exclude": {
      "uid": [1, 2, 3]
    }
  },
  "topic": "some-topic"
}
```

### Basic Usage (backend - PHP)

Sending a message to all authenticated users when a node is created (example from driplet_notify module):

```php
function driplet_notify_node_insert(NodeInterface $node) {
  $driplet = \Drupal::service('driplet.service');

  $message = $driplet->createMessage()
    ->setMessage([
      'type' => 'notify-persist',
      'content' => t('New @type page <a href="@url">@title</a> has been created just now.', [
        '@type' => $node->getType(),
        '@url' => $node->toUrl()->toString(),
        '@title' => $node->getTitle(),
      ]),
    ])
    ->setTopic('driplet-notify')
    ->include()
    ->setTarget('roles', 'authenticated');

  $driplet->sendMessage($message);
}
```

### Basic Usage (frontend - JavaScript)

```javascript
// Get the DripletClient instance
const client = DripletClient.getInstance(wsEndpoint, jwtEndpoint);
// Subscribe to a topic
client.setTopics(['some-topic']);

// Handle received messages
client.onMessage((data) => {
  // Return if the message is not for our topic
  if (data.topic !== 'some-topic') return;

  // Handle the message
  console.log('Received message:', data.message);
});
``````

## Development with DDEV

To use Driplet with DDEV follow these steps:

1. Have Drupal all set up with DDEV
2. Inside `.ddev` directory create a new file `docker-compose.driplet.yml` with the following content:


```yaml
services:
  driplet:
    image: make0x20/driplet:latest
    container_name: ddev-${DDEV_SITENAME}-driplet
    labels:
      com.ddev.site-name: ${DDEV_SITENAME}
      com.ddev.approot: ${DDEV_APPROOT}
    expose:
      - "4719"
    environment:
      - VIRTUAL_HOST=driplet.${DDEV_HOSTNAME}
      - HTTP_EXPOSE=4718:4719
      - HTTPS_EXPOSE=4719:4719
    volumes:
      - ./driplet:/app/config
    command: ["./driplet", "-config", "config/config.toml"]
```
3. Restart DDEV. It should create a new directory inside `.ddev` called `driplet` with default `config.toml`.
  - You can quickly check Driplet status by running `ddev logs -s driplet` or visiting `https://driplet.{your-ddev-site-name}.ddev.site:4719`.
4. For development purposes, it's usually okay to roll with the default config.
5. Configure Driplet module to use the following configuration:
    - API Host: `driplet:4719`
    - Use SSL for API: `False`
    - WebSocket Host: `driplet.{your-ddev-site-name}.ddev.site:4719`
    - Use SSL for WebSocket: `True`
    - Service Name: `default`
    - JWT Secret: `change-this-jwt-secret` (this is the default secret in Driplet - CHANGE WHEN RUNNING IN PRODUCTION)
    - API Secret: `change-this-api-secret` (this is the default secret in Driplet - CHANGE WHEN RUNNING IN PRODUCTION)
6. Save the configuration and you should be good to go.
7. Test by enabling either `driplet_notify` or `driplet_log` module to see it in action.
