(function() {
  'use strict';

  // Private store for client instances (one per endpoint)
  const clients = new Map();

  class DripletClient {
    static getInstance(wsEndpoint, jwtEndpoint) {
      if (!clients.has(wsEndpoint)) {
        const client = new DripletClient(wsEndpoint, jwtEndpoint);
        clients.set(wsEndpoint, client);

        // Connect immediately when creating the instance
        client.connect().catch(error => {
          console.error('Failed to connect:', error);
        });
      }
      return clients.get(wsEndpoint);
    }

    constructor(wsEndpoint, jwtEndpoint) {
      if (clients.has(wsEndpoint)) {
        throw new Error('Use DripletClient.getInstance()');
      }

      this.wsEndpoint = wsEndpoint;
      this.jwtEndpoint = jwtEndpoint;
      this.ws = null;
      this.topics = new Set();
      this.messageCallbacks = new Set();
      this.reconnectAttempts = 0;
      this.maxReconnectAttempts = 5;
      this.reconnectDelay = 1000;
      this.connecting = null; // Promise to track connection state
    }

    setTopics(topics) {
      // Merge new topics with existing ones instead of replacing
      topics.forEach(topic => this.topics.add(topic));

      // if already connected, subscribe to new topics
      if (this.isConnected()) {
        topics.forEach(topic => this.subscribe(topic));
      }
    }

    // subscribe to a topic
    subscribe(topic) {
      if (!this.isConnected()) {
        this.topics.add(topic);
        return;
      }

      this.topics.add(topic);
      this._sendMessage({
        type: 'subscribe',
        topic
      });
    }

    // unsubscribe from a topic
    unsubscribe(topic) {
      if (!this.isConnected()) {
        this.topics.delete(topic);
        return;
      }

      this.topics.delete(topic);
      this._sendMessage({
        type: 'unsubscribe',
        topic
      });
    }

    async connect() {
      // If already connecting, return the existing promise
      if (this.connecting) {
        return this.connecting;
      }

      // If already connected, return
      if (this.isConnected()) {
        return Promise.resolve();
      }

      // Create new connection promise
      this.connecting = new Promise(async (resolve, reject) => {
        try {
          const token = await this._fetchToken();
          await this._establishConnection(token);
          resolve();
        } catch (error) {
          this._handleReconnect();
          reject(error);
        } finally {
          this.connecting = null;
        }
      });

      return this.connecting;
    }

    onMessage(callback) {
      this.messageCallbacks.add(callback);
      return () => this.messageCallbacks.delete(callback); // Return cleanup function
    }

    disconnect() {
      if (this.ws) {
        this.ws.close();
        this.ws = null;
      }
    }

    isConnected() {
      return this.ws?.readyState === WebSocket.OPEN;
    }

    // Private methods
    async _fetchToken() {
      const response = await fetch(this.jwtEndpoint, {
        credentials: 'same-origin',
        headers: { 'Accept': 'application/json' },
      });

      if (!response.ok) {
        throw new Error('Failed to fetch JWT');
      }

      const data = await response.json();
      return data.token;
    }

    async _establishConnection(token) {
      return new Promise((resolve, reject) => {
        const wsUrl = `${this.wsEndpoint}?token=${encodeURIComponent(token)}`;
        this.ws = new WebSocket(wsUrl);

        this.ws.onopen = () => {
          console.log('Connected to Driplet');
          this.reconnectAttempts = 0;
          this.reconnectDelay = 1000;

          // Subscribe to all topics
          this.topics.forEach(topic => {
            this._sendMessage({
              type: 'subscribe',
              topic
            });
          });

          resolve();
        };

        // Handle incoming messages
        this.ws.onmessage = (event) => {
          try {
            const message = JSON.parse(event.data);
            this.messageCallbacks.forEach(callback => {
              try {
                callback(message);
              } catch (e) {
                console.error('Error in message callback:', e);
              }
            });
          } catch (e) {
            console.error('Error processing message:', e);
          }
        };

        // Handle disconnection
        this.ws.onclose = () => {
          console.log('Disconnected from Driplet');
          this.ws = null;
          this._handleReconnect();
        };

        // Handle errors
        this.ws.onerror = (error) => {
          console.error('WebSocket error:', error);
          reject(error);
        };
      });
    }

    _handleReconnect() {
      if (this.reconnectAttempts >= this.maxReconnectAttempts) {
        console.error('Max reconnection attempts reached');
        return;
      }

      setTimeout(() => {
        this.reconnectAttempts++;
        this.reconnectDelay *= 2;
        this.connect().catch(() => { }); // Silently catch error as it's already logged
      }, this.reconnectDelay);
    }

    _sendMessage(message) {
      if (this.isConnected()) {
        this.ws.send(JSON.stringify(message));
      }
    }
  }

  window.DripletClient = DripletClient;
})();
