<?php

namespace Drupal\driplet_log\Logger;

use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Logger\LogMessageParserInterface;
use Drupal\Core\Logger\RfcLoggerTrait;
use Drupal\driplet\Service\DripletService;
use Psr\Log\LoggerInterface;

/**
 * Logs events through Driplet WebSocket service.
 */
class DripletLogger implements LoggerInterface {
  use RfcLoggerTrait;
  use DependencySerializationTrait;

  /**
   * The message's placeholders parser.
   *
   * @var \Drupal\Core\Logger\LogMessageParserInterface
   */
  protected $parser;

  /**
   * The Driplet service.
   *
   * @var \Drupal\driplet\Service\DripletService
   */
  protected $dripletService;

  /**
   * Constructs a DripletLogger object.
   *
   * @param \Drupal\Core\Logger\LogMessageParserInterface $parser
   *   The parser to use when extracting message variables.
   * @param \Drupal\driplet\Service\DripletService $driplet_service
   *   The Driplet service.
   */
  public function __construct(LogMessageParserInterface $parser, DripletService $driplet_service) {
    $this->parser = $parser;
    $this->dripletService = $driplet_service;
  }

  /**
   * {@inheritdoc}
   */
  public function log($level, string|\Stringable $message, array $context = []): void {
    // Skip driplet-related messages to prevent loops.
    if ($context['channel'] === 'driplet_log' || $context['channel'] === 'driplet_notify') {
      return;
    }

    $message_placeholders = $this->parser->parseMessagePlaceholders($message, $context);
    $message = strtr($message, $message_placeholders);

    try {
      $driplet_message = $this->dripletService->createMessage()
        ->setMessage([
          'severity' => $level,
          'type' => mb_substr($context['channel'], 0, 64),
          'message' => strip_tags($message),
          'timestamp' => $context['timestamp'],
          'uid' => $context['uid'],
          'link' => $context['link'] ?? NULL,
        ])
        ->setTopic('driplet-log')
        ->include()
        ->setTarget('roles', 'administrator');

      $this->dripletService->sendMessage($driplet_message);
    }
    catch (\Exception $e) {
      \Drupal::logger('driplet')->error('Failed to send Driplet message: @message', ['@message' => $e->getMessage()]);
    }
  }

}
