<?php

namespace Drupal\driplet\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configuration form for Driplet settings.
 */
class DripletSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['driplet.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'driplet_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('driplet.settings');

    $form['host_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Driplet Host Settings'),
    ];

    $form['host_settings']['api'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('API Connection - Drupal to Driplet'),
    ];

    $form['host_settings']['api']['driplet_api_host'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Host'),
      '#description' => $this->t('The hostname for REST API calls - format host:port (e.g., driplet:4719 or driplet.example.com)'),
      '#placeholder' => 'driplet:4719',
      '#default_value' => $config->get('driplet_api_host'),
      '#required' => TRUE,
    ];

    $form['host_settings']['api']['driplet_api_use_ssl'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use SSL for API'),
      '#description' => $this->t('Enable HTTPS for API calls. Usually disabled for internal communication.'),
      '#default_value' => $config->get('driplet_api_use_ssl'),
    ];

    $form['host_settings']['websocket'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('WebSocket Connection - Browsers to Driplet'),
    ];

    $form['host_settings']['websocket']['driplet_websocket_host'] = [
      '#type' => 'textfield',
      '#title' => $this->t('WebSocket Host'),
      '#description' => $this->t('The hostname for WebSocket connections - format host:port (e.g., driplet:4719 or driplet.example.com)'),
      '#placeholder' => 'driplet.drupal-test.ddev.site:4719',
      '#default_value' => $config->get('driplet_websocket_host'),
      '#required' => TRUE,
    ];

    $form['host_settings']['websocket']['driplet_websocket_use_ssl'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use SSL for WebSocket'),
      '#description' => $this->t('Enable WSS for WebSocket connections. Usually enabled for browser connections.'),
      '#default_value' => $config->get('driplet_websocket_use_ssl'),
    ];

    $form['service_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Service Settings'),
    ];

    $form['service_settings']['driplet_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Service Name'),
      '#description' => $this->t('The service name used in endpoints (e.g., default)'),
      '#placeholder' => 'default',
      '#default_value' => $config->get('driplet_name'),
      '#required' => TRUE,
    ];

    $form['security'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Security Settings'),
    ];

    $form['security']['driplet_jwt_secret'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Driplet JWT Secret'),
      '#description' => $this->t('Secret key for JWT authentication. Generate using: openssl rand -hex 32'),
      '#placeholder' => 'Example: HJ8iZzF3kL9jNb5sXq2mP4wRvY6tUc7x...',
      '#default_value' => $config->get('driplet_jwt_secret'),
      '#required' => TRUE,
    ];

    $form['security']['driplet_api_secret'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Driplet API Secret'),
      '#description' => $this->t('Secret key for message signing. Generate using: openssl rand -hex 32'),
      '#placeholder' => 'Example: Ky7mWx4nJh9fVb2tPc5sZq3rL8vN6...',
      '#default_value' => $config->get('driplet_api_secret'),
      '#required' => TRUE,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $apiUseSSL = $form_state->getValue('driplet_api_use_ssl');
    $wsUseSSL = $form_state->getValue('driplet_websocket_use_ssl');
    $apiHost = $form_state->getValue('driplet_api_host');
    $wsHost = $form_state->getValue('driplet_websocket_host');
    $name = $form_state->getValue('driplet_name');

    // Set protocols based on individual SSL settings.
    $apiProtocol = $apiUseSSL ? 'https' : 'http';
    $wsProtocol = $wsUseSSL ? 'wss' : 'ws';

    $this->configFactory->getEditable('driplet.settings')
      ->set('driplet_api_host', $apiHost)
      ->set('driplet_websocket_host', $wsHost)
      ->set('driplet_api_use_ssl', $apiUseSSL)
      ->set('driplet_websocket_use_ssl', $wsUseSSL)
      ->set('driplet_name', $name)
      ->set('driplet_jwt_secret', $form_state->getValue('driplet_jwt_secret'))
      ->set('driplet_api_secret', $form_state->getValue('driplet_api_secret'))

     // Set endpoints.
      ->set('driplet_api_endpoint', sprintf('%s://%s/api/%s/message', $apiProtocol, $apiHost, $name))
      ->set('driplet_websocket_endpoint', sprintf('%s://%s/ws/%s', $wsProtocol, $wsHost, $name))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
